@@ -7,6 +7,8 @@ import pendulum
 from pypika.terms import LiteralValue
 from pypika import Criterion
 
+from metrics_layer.core.exceptions import QueryError
+
 from .base import MetricsLayerBase
 
 
@@ -97,6 +99,7 @@ class FilterInterval(str, Enum):
             return cls.unknown
 
 
+# TODO this does not account for a changed "week_start_date"
 class Filter(MetricsLayerBase):
     def __init__(self, definition: dict = {}) -> None:
         self.validate(definition)
@@ -106,7 +109,7 @@ class Filter(MetricsLayerBase):
         required_keys = ["field", "value"]
         for k in required_keys:
             if k not in definition:
-                raise ValueError(f"Filter missing required key '{k}' The filter passed was {definition}")
+                raise QueryError(f"Filter missing required key '{k}' The filter passed was {definition}")
 
     def filter_dict(self, json_safe: bool = False) -> list:
         filter_dict = self._filter_dict(self.field, self.value)
@@ -162,7 +165,9 @@ class Filter(MetricsLayerBase):
         if date_condition == "today":
             expression = MetricsLayerFilterExpressionType.GreaterOrEqualThan
             cleaned_value = Filter._start_date(lag=0, date_part=FilterInterval.day)
-            return [(expression, cleaned_value)]
+            end_expression = MetricsLayerFilterExpressionType.LessOrEqualThan
+            end_value = Filter._end_date(lag=0, date_part=FilterInterval.day)
+            return [(expression, cleaned_value), (end_expression, end_value)]
 
         if date_condition == "yesterday":
             start_expression = MetricsLayerFilterExpressionType.GreaterOrEqualThan
@@ -208,10 +213,10 @@ class Filter(MetricsLayerBase):
         start_value = Filter._start_date(lag=lag, date_part=date_part)
         result = [(start_expression, start_value)]
 
-        if n == "last":
-            end_expression = MetricsLayerFilterExpressionType.LessOrEqualThan
-            end_value = Filter._end_date(lag=lag, date_part=date_part)
-            result.append((end_expression, end_value))
+        end_lag = lag if n == "last" else 0
+        end_expression = MetricsLayerFilterExpressionType.LessOrEqualThan
+        end_value = Filter._end_date(lag=end_lag, date_part=date_part)
+        result.append((end_expression, end_value))
 
         return result
 
@@ -336,7 +341,7 @@ class Filter(MetricsLayerBase):
                 cleaned_value = [f"{category[1:].strip()}" for category in value.split(", ")]
 
             elif any(category[0] == "-" for category in value.split(", ")):
-                raise ValueError("Invalid filter some elements are negated with '-' and some are not")
+                raise QueryError("Invalid filter some elements are negated with '-' and some are not")
 
             else:
                 expression = MetricsLayerFilterExpressionType.IsIn
@@ -462,4 +467,4 @@ class Filter(MetricsLayerBase):
         try:
             return criterion_strategies[expression_type](field)
         except KeyError:
-            raise NotImplementedError(f"Unknown filter expression_type: {expression_type}.")
+            raise QueryError(f"Unknown filter expression_type: {expression_type}.")
